@@ -12,12 +12,13 @@
 #'   Currently, the following families are supported:
 #'   \code{gaussian}, \code{student}, \code{binomial}, 
 #'   \code{bernoulli}, \code{poisson}, \code{negbinomial}, 
-#'   \code{geometric}, \code{Gamma}, \code{lognormal}, \code{inverse.gaussian}, 
+#'   \code{geometric}, \code{Gamma}, \code{lognormal}, 
+#'   \code{exgaussian}, \code{wiener}, \code{inverse.gaussian}, 
 #'   \code{exponential}, \code{weibull}, \code{Beta}, \code{von_mises},
 #'   \code{categorical}, \code{cumulative}, \code{cratio}, \code{sratio}, 
 #'   \code{acat}, \code{hurdle_poisson}, \code{hurdle_negbinomial}, 
 #'   \code{hurdle_gamma}, \code{hurdle_lognormal}, 
-#'   \code{zero_inflated_binomial}, \cr \code{zero_inflated_beta},
+#'   \code{zero_inflated_binomial}, \code{zero_inflated_beta},
 #'   \code{zero_inflated_negbinomial}, 
 #'   and \code{zero_inflated_poisson}.
 #' @param link A specification for the model link function. 
@@ -39,6 +40,12 @@
 #'   leads to ordinal regression. Families \code{Gamma}, \code{weibull}, 
 #'   \code{exponential}, \code{lognormal}, and \code{inverse.gaussian} can be used 
 #'   (among others) for survival regression.
+#'   Family \code{exgaussian} ('exponentially modified Gaussian') is especially
+#'   suited to model reaction times and the \code{wiener} family provides
+#'   an implementation of the Wiener diffusion model. For this family,
+#'   the main formula predicts the drift parameter 'delta' and
+#'   all other parameters are modeled as auxiliary parameters 
+#'   (see \code{\link[brms:brmsformula]{brmsformula}} for details).
 #'   Families \code{hurdle_poisson}, \code{hurdle_negbinomial}, \code{hurdle_gamma}, 
 #'   \code{hurdle_lognormal}, \code{zero_inflated_poisson},
 #'   \code{zero_inflated_negbinomial}, \code{zero_inflated_binomial}, and
@@ -50,7 +57,7 @@
 #'   models cannot be reasonably fitted for data containing zeros in the response.
 #'   
 #'   In the following, we list all possible links for each family.
-#'   The families \code{gaussian}, and \code{student},
+#'   The families \code{gaussian}, \code{student}, and \code{exgaussian}
 #'   accept the links (as names) \code{identity}, \code{log}, and \code{inverse};
 #'   families \code{poisson}, \code{negbinomial}, and \code{geometric} the links 
 #'   \code{log}, \code{identity}, and \code{sqrt}; 
@@ -58,7 +65,7 @@
 #'   \code{cumulative}, \code{cratio}, \code{sratio}, and \code{acat} 
 #'   the links \code{logit}, \code{probit}, \code{probit_approx}, 
 #'   \code{cloglog}, and \code{cauchit}; 
-#'   family \code{categorical} the link \code{logit}; 
+#'   family \code{categorical} the link \code{logit};
 #'   families \code{Gamma}, \code{weibull}, and \code{exponential} 
 #'   the links \code{log}, \code{identity}, and \code{inverse};
 #'   family \code{lognormal} the links \code{identity} and \code{inverse};
@@ -67,7 +74,7 @@
 #'   families \code{hurdle_poisson}, \code{hurdle_negbinomial},
 #'   \code{hurdle_gamma}, \code{zero_inflated_poisson}, and
 #'   \code{zero_inflated_negbinomial} the link \code{log};
-#'   family \code{hurdle_lognormal} the link \code{identity}.
+#'   families \code{wiener} and \code{hurdle_lognormal} the link \code{identity}.
 #'   The first link mentioned for each family is the default.     
 #'   
 #'   Please note that when calling the \code{\link[stats:family]{Gamma}} 
@@ -120,8 +127,8 @@ brmsfamily <- function(family, link = NULL) {
     "gaussian", "student", "lognormal", 
     "binomial", "bernoulli", "categorical", 
     "poisson", "negbinomial", "geometric", 
-    "gamma", "weibull", "exponential", 
-    "inverse.gaussian", "beta", "von_mises",
+    "gamma", "weibull", "exponential", "exgaussian",
+    "inverse.gaussian", "wiener", "beta", "von_mises",
     "cumulative", "cratio", "sratio", "acat",
     "hurdle_poisson", "hurdle_negbinomial", "hurdle_gamma",
     "hurdle_lognormal", "zero_inflated_poisson", 
@@ -133,7 +140,7 @@ brmsfamily <- function(family, link = NULL) {
   }
   
   # check validity of link
-  if (is.linear(family)) {
+  if (is.linear(family) || family %in% "exgaussian") {
     ok_links <- c("identity", "log", "inverse")
   } else if (family == "inverse.gaussian") {
     ok_links <- c("1/mu^2", "inverse", "identity", "log")
@@ -150,9 +157,9 @@ brmsfamily <- function(family, link = NULL) {
     ok_links <- c("log", "identity", "inverse")
   } else if (is.lognormal(family)) {
     ok_links <- c("identity", "inverse")
-  } else if (family %in% "hurdle_lognormal") {
+  } else if (family %in% c("hurdle_lognormal", "wiener")) {
     ok_links <- c("identity")
-  } else if (family %in% "von_mises") {
+  } else if (family %in% c("von_mises")) {
     ok_links <- c("tan_half")
   } else if (is.hurdle(family) || is.zero_inflated(family)) {
     # does not include zi_binomial, zi_beta, or hu_lognormal
@@ -233,6 +240,20 @@ weibull <- function(link = "log") {
   .brmsfamily("weibull", link = link, slink = slink)
 }
 
+#' @rdname brmsfamily
+#' @export
+exgaussian <- function(link = "identity") {
+  slink <- substitute(link)
+  .brmsfamily("exgaussian", link = link, slink = slink)
+}
+
+#' @rdname brmsfamily
+#' @export
+wiener <- function(link = "identity") {
+  slink <- substitute(link)
+  .brmsfamily("wiener", link = link, slink = slink)
+}
+
 #' @rdname brmsfamily
 #' @export
 Beta <- function(link = "logit") {
@@ -423,6 +444,22 @@ is.lognormal <- function(family) {
   family %in% c("lognormal")
 }
 
+is.exgaussian <- function(family) {
+  # indicate if family is exgaussian
+  if (is(family, "family")) {
+    family <- family$family
+  }
+  family %in% c("exgaussian")
+}
+
+is.wiener <- function(family) {
+  # indicate if family is the wiener diffusion model
+  if (is(family, "family")) {
+    family <- family$family
+  }
+  family %in% c("wiener")
+}
+
 is.count <- function(family) {
   # indicate if family is for a count model
   if (is(family, "family")) {
@@ -478,7 +515,7 @@ is.mv <- function(family, response = NULL) {
   is_mv <- nresp > 1L && is.linear(family) || is.categorical(family) || 
            nresp == 2L && is.forked(family)
   if (nresp > 1L && !is_mv) {
-    stop("Invalid multivariate model", call. = FALSE)
+    stop2("Invalid multivariate model")
   }
   is_mv
 }
@@ -489,8 +526,9 @@ use_real <- function(family) {
     family <- family$family
   }
   is.linear(family) || is.skewed(family) || 
-    family %in% c("lognormal", "inverse.gaussian", "beta", "von_mises",
-                  "zero_inflated_beta", "hurdle_gamma", "hurdle_lognormal")
+    family %in% c("lognormal", "exgaussian", "inverse.gaussian", "beta", 
+                  "von_mises", "zero_inflated_beta", "hurdle_gamma", 
+                  "hurdle_lognormal", "wiener")
 }
 
 use_int <- function(family) {
@@ -553,8 +591,16 @@ has_kappa <- function(family) {
   family %in% c("von_mises")
 }
 
-has_sigma <- function(family, effects = NULL, autocor = cor_arma(),
-                      incmv = FALSE) {
+has_beta <- function(family) {
+  # indicate if family needs a kappa parameter
+  if (is(family, "family")) {
+    family <- family$family
+  }
+  family %in% c("exgaussian")
+}
+
+has_sigma <- function(family, effects = NULL, 
+                      autocor = cor_arma(), incmv = FALSE) {
   # indicate if the model needs a sigma parameter
   # Args:
   #  family: model family
@@ -564,10 +610,21 @@ has_sigma <- function(family, effects = NULL, autocor = cor_arma(),
   if (is(family, "family")) {
     family <- family$family
   }
-  is_lognormal <- family %in% c("lognormal", "hurdle_lognormal")
-  has_se <- !is.null(effects$se)
-  out <- (is.linear(family) || is_lognormal) && 
-         (!has_se || use_cov(autocor)) && !is(autocor, "cov_fixed")
+  is_ln_eg <- family %in% c("lognormal", "hurdle_lognormal", "exgaussian")
+  if (is.formula(effects$se)) {
+    # call .se without evaluating the x argument 
+    cl <- rhs(effects$se)[[2]]
+    cl[[1]] <- quote(.se_no_data)
+    se_only <- isFALSE(attr(eval(cl), "sigma")) 
+    if (se_only && use_cov(autocor)) {
+      stop2("Please set argument 'sigma' of function 'se' ",  
+            "to TRUE when modeling ARMA covariance matrices.")
+    }
+  } else {
+    se_only <- FALSE
+  }
+  out <- (is.linear(family) || is_ln_eg) && 
+           !se_only && !is(autocor, "cov_fixed")
   if (!incmv) {
     is_multi <- is.linear(family) && length(effects$response) > 1L
     out <- out && !is_multi
@@ -575,7 +632,7 @@ has_sigma <- function(family, effects = NULL, autocor = cor_arma(),
   out
 }
 
-allows_cse <- function(family) {
+allows_cs <- function(family) {
   # checks if category specific effects are allowed
   if (is(family, "family")) {
     family <- family$family
