@@ -255,6 +255,24 @@ loglik_inverse.gaussian <- function(i, draws, data = data.frame()) {
   weight_loglik(out, i = i, data = draws$data)
 }
 
+loglik_exgaussian <- function(i, draws, data = data.frame()) {
+  args <- list(mu = ilink(get_eta(draws, i), draws$f$link), 
+               sigma = get_sigma(draws$sigma, data = draws$data, i = i),
+               beta = get_auxpar(draws$beta, i = i))
+  out <- censor_loglik(dist = "exgauss", args = args, i = i, data = draws$data)
+  weight_loglik(out, i = i, data = draws$data)
+}
+
+loglik_wiener <- function(i, draws, data = data.frame()) {
+  args <- list(delta = ilink(get_eta(draws, i), draws$f$link), 
+               alpha = get_auxpar(draws$bs, i = i),
+               tau = get_auxpar(draws$ndt, i = i),
+               beta = get_auxpar(draws$bias, i = i),
+               resp = draws$data[["dec"]][i])
+  out <- do.call("dWiener", c(draws$data$Y[i], args, log = TRUE))
+  weight_loglik(out, i = i, data = draws$data)
+}
+
 loglik_beta <- function(i, draws, data = data.frame()) {
   mu <- ilink(get_eta(draws, i), draws$f$link)
   phi <- get_auxpar(draws$phi, i)
