@@ -1,6 +1,7 @@
 test_that("all S3 methods have reasonable ouputs", {
   fit1 <- brms:::rename_pars(brms:::brmsfit_example1)
   fit2 <- brms:::rename_pars(brms:::brmsfit_example2)
+  fit3 <- brms:::rename_pars(brms:::brmsfit_example3)
   # test S3 methods in alphabetical order
   # as.data.frame
   ps <- as.data.frame(fit1)
@@ -15,6 +16,8 @@ test_that("all S3 methods have reasonable ouputs", {
   mc <- as.mcmc(fit1)
   expect_equal(length(mc), chains)
   expect_equal(dim(mc[[1]]), c(Nsamples(fit1) / chains, length(parnames(fit1))))
+  mc <- as.mcmc(fit1, combine_chains = TRUE)
+  expect_equal(dim(mc), c(Nsamples(fit1), length(parnames(fit1))))
   # test assumes thin = 1
   expect_equal(dim(as.mcmc(fit1, inc_warmup = TRUE)[[1]]), 
                c(fit1$fit@sim$iter, length(parnames(fit1))))
@@ -96,7 +99,12 @@ test_that("all S3 methods have reasonable ouputs", {
   # expect_output(print(loo_compare3), "Weights")
   
   loo2 <- SW(LOO(fit2, cores = 1))
-  expect_true(is.numeric(loo1[["looic"]]))
+  expect_true(is.numeric(loo2[["looic"]]))
+  
+  loo3 <- SW(LOO(fit3, cores = 1))
+  expect_true(is.numeric(loo3[["looic"]]))
+  loo3 <- SW(LOO(fit3, pointwise = TRUE, cores = 1))
+  expect_true(is.numeric(loo3[["looic"]]))
   # marginal_effects (the related plot method is tested in tests.plots)
   expect_equal(nrow(marginal_effects(fit1)[[2]]), 100)
   mdata = data.frame(Age = c(-0.3, 0, 0.3), count = c(10, 20, 30), 
@@ -136,7 +144,7 @@ test_that("all S3 methods have reasonable ouputs", {
   expect_equal(nobs(fit1), nrow(epilepsy))
   # parnames 
   expect_equal(parnames(fit1)[c(1, 8, 9, 13, 15, 17, 27, 35, 38, 46)],
-               c("b_Intercept", "bm_Exp", "ar[1]", "cor_visit__Intercept__Trt", 
+               c("b_Intercept", "bmo_Exp", "ar[1]", "cor_visit__Intercept__Trt", 
                  "nu", "simplex_Exp[2]", "r_visit[4,Trt]", "s_sAge_1[8]", 
                  "prior_sd_visit", "lp__"))
   expect_equal(parnames(fit2)[c(1, 4, 6, 7, 9, 71, 129)],
@@ -162,6 +170,7 @@ test_that("all S3 methods have reasonable ouputs", {
   expect_true(is(pp_check(fit1, "error_binned"), "ggplot"))
   ribbon_plot <- pp_check(fit1, "ribbon_grouped", group = "visit", x = "Age")
   expect_true(is(ribbon_plot, "ggplot"))
+  expect_true(is(pp_check(fit3), "ggplot"))
   expect_true(is(pp_check(fit2, "ribbon", x = "Trt"), "ggplot"))
   expect_error(pp_check(fit1, "wrong_type"))
   expect_error(pp_check(fit2, "violin_grouped"), "group")
@@ -203,7 +212,7 @@ test_that("all S3 methods have reasonable ouputs", {
   expect_output(SW(print(fit1)), "Group-Level Effects:")
   # prior_samples
   prs1 <- prior_samples(fit1)
-  prior_names <- c("sds_sAge_1", "nu", "sd_visit", "b", "bm", 
+  prior_names <- c("sds_sAge_1", "nu", "sd_visit", "b", "bmo", 
                    paste0("simplex_Exp[", 1:4, "]"), "cor_visit")
   expect_equal(dimnames(prs1),
                list(as.character(1:Nsamples(fit1)), prior_names))
@@ -241,7 +250,7 @@ test_that("all S3 methods have reasonable ouputs", {
   # standata
   expect_equal(names(standata(fit1)),
                c("N", "Y", "nb_1", "knots_1", "Zs_1_1", "K", "X", 
-                 "Km", "Xm", "Jm", "con_simplex_1", "Z_1_1", "Z_1_2", 
+                 "Kmo", "Xmo", "Jmo", "con_simplex_1", "Z_1_1", "Z_1_2", 
                  "offset", "K_sigma", "X_sigma", "J_1", "N_1", "M_1", 
                  "NC_1", "tg", "Kar", "Kma", "Karma", "prior_only"))
   expect_equal(names(standata(fit2)),
@@ -326,4 +335,10 @@ test_that("all S3 methods have reasonable ouputs", {
   waic_pointwise <- WAIC(fit2, pointwise = TRUE)
   expect_equal(waic2, waic_pointwise)
   expect_warning(WAIC(fit1, fit2), "Model comparisons are most likely invalid")
+  
+  # test diagnostic convenience functions
+  expect_true(is(log_posterior(fit1), "data.frame"))
+  expect_true(is(nuts_params(fit1), "data.frame"))
+  expect_true(is(rhat(fit1), "numeric"))
+  expect_true(is(neff_ratio(fit1), "numeric"))
 })
\ No newline at end of file
