@@ -8,25 +8,18 @@ test_that("plotting functions don't throw unexpected errors", {
   expect_error(plot(fit, pars = "123"),  "No valid parameters selected")
   
   # stanplot.brmsfit
-  expect_silent(p <- stanplot(fit, quiet = TRUE))
-  expect_silent(p <- stanplot(fit, pars = "^b", quiet = TRUE))
-  expect_silent(p <- stanplot(fit, type = "trace", quiet = TRUE))
-  expect_silent(p <- stanplot(fit, type = "hist", quiet = TRUE))
-  expect_silent(p <- stanplot(fit, type = "dens", quiet = TRUE))
-  expect_silent(p <- stanplot(fit, type = "scat", quiet = TRUE,
+  expect_silent(p <- stanplot(fit))
+  expect_silent(p <- stanplot(fit, pars = "^b"))
+  expect_silent(p <- stanplot(fit, type = "trace", pars = "^b_"))
+  expect_silent(p <- stanplot(fit, type = "hist", pars = "^sd_"))
+  expect_silent(p <- stanplot(fit, type = "dens"))
+  expect_silent(p <- stanplot(fit, type = "scatter",
                               pars = parnames(fit)[2:3], 
                               exact_match = TRUE))
-  #expect_silent(p <- stanplot(fit, type = "diag", quiet = TRUE))
-  expect_silent(p <- stanplot(fit, type = "rhat", pars = "^b_",
-                              quiet = TRUE))
-  expect_silent(p <- stanplot(fit, type = "ess", quiet = TRUE))
-  expect_silent(p <- stanplot(fit, type = "mcse", quiet = TRUE))
-  expect_silent(p <- stanplot(fit, type = "ac", quiet = TRUE))
-  # warning occurs somewhere in rstan
-  expect_silent(SW(stanplot(fit, type = "par", 
-                            pars = "^b_Intercept$")))
-  expect_error(p <- stanplot(fit, type = "par", pars = "^b_"),
-               "'stan_par' expects a single parameter name")
+  expect_silent(p <- stanplot(fit, type = "rhat", pars = "^b_"))
+  expect_silent(p <- stanplot(fit, type = "neff"))
+  expect_silent(p <- stanplot(fit, type = "acf"))
+  expect_silent(p <- stanplot(fit, type = "nuts_divergence"))
   expect_error(stanplot(fit, type = "density"), "Invalid plot type")
   
   # pairs.brmsfit
